//====== Promos =====================================================
//  Demonstracni program pro obsluhu maticove klavesnice 8x8.
//  Pro pripojeni se pouziva KIO-PIO, kanal B - buzeni sloupcu, kanal A
//  - cteni radku. Pro klavesnice s mensim poctem klaves je mozne porty
//  vyuzit pouze castecne. 
//  Perioda skanovani je rizena internim casovacem Z180 PRT1.
//  Skanovani klavesnice probiha pod prerusenim. Pri opakovane identifikaci 
//  stejne klavesy (pocet nutnych shod je urcen konstantou KB_CP) je klavesa
//  povazovana za stlacenou.  
//  Globalni promenna kb_key obsahuje aktualni kod po celou dobu stlaceni 
//  klavesy. Promenna kb_code obsahuje kod naposledy stlacene klavesy, 
//  program obsluhy musi zajistit vynulovani kb_code po zpracovani. Pri
//  trvalem stlaceni klavesy je po probehnuti prodlevy do kb_code periodicky 
//  znovu nastavovan kod (obdobne jako v PC), prodleva a perioda opakovani 
//  jsou urceny konstantami REPDELAY a REPRATE.
 
#use "compact.lib"	// pro SBPS-02
//#use "pmscpu.lib"	// pro SBPS-01

#define REPDELAY  50         // prodleva 
#define REPRATE   20         // opakovani
#define RATE    2304         // 200 Hz - 5 ms
#define KB_CP      4         // pocet stejnych vyhodnoceni

// globalni promenne pro predavani kodu klavesy
char kb_key;   // kod prave stlacene klavesy (00 pokud neni stlaceno nic)
char kb_code;  // kod posledni stlacene klavesy (po prevzeti nutno vynulovat) 

// pracovni promenne
char kb_count, // pocitadlo shody
     kb_min,   // minuly stav radku
     kb_rep,   // priznak opakovani
     kb_rc;    // citac prodlevy

init_keyboard () {
   outport (_PIOCA, 0xFF); 
   outport (_PIOCA, 0xFF);    // port A vstup
   outport (_PIOCB, 0xFF); 
   outport (_PIOCB, 0x00);    // port B vystup
   kb_count=KB_CP; kb_key=kb_code=0; kb_min=0xFF;
   prt1_init(RATE);
}

#INT_VEC PRT1_VEC timer_int
#asm
timer_int::  push af
             push bc
             push hl
             in0 a, (TCR)
             in0 a, (TMDR1L)    ; clear TIF
             call kbd_test
             pop hl
             pop bc
             pop af
             ei
             ret
      
kbd_test:    xor a, a
             out0 (_PIODB), a   ; buzeni vsech sloupcu najednou
             in0 a, (_PIODA)    ; cteni vsech radku
             ld hl,kb_min
             cp a,(hl)          ; porovnani s minulym kodem
             jr nz,kb_change    ; zmena
             ld hl,kb_count     ; pocitadlo shody
             dec (hl)
             ret nz             ; jeste je dost casu
             cp a,0FFh          ; FF pokud neni nic stlaceno
             jp nz,kb_xscan     ; neco nalezeno
             xor a,a
             ld (kb_key),a      ;
             ld (kb_rep),a      ; vynulovani priznaku
             ret
kb_change:   ld (hl),a          ; novy kod - zmena
             xor a,a
             ld a,REPDELAY
             ld (kb_rc),a       ; doba na prodlevu
             ld a,KB_CP
             ld (kb_count),a    ; pocitadlo shody od zacatku
             ret

kb_xscan:    call kb_scan       ; prohledani klavesnice, scan code v A
             ld (kb_key),a      ; ulozeni okamziteho scan code
             ld a,(kb_code)
             or a,a
             jr nz,kb_exCP      ; jeste nebyl odebran
             ld a,(kb_rep)
             ld b,a
             ld a,(kb_key)      ;
             cp b
             jr z,kb_repx       ; byl to ten samy bez uvolneni klavesy
kb_setcode:  ld (kb_code),a     ; ulozeni posledni klavesy
             ld (kb_rep),a      ; priznak pro opakovani
             jp kb_exCP
kb_repx:     ld hl,kb_rc        ; pocitadlo repeatu
             dec (hl)           ;
             jp nz,kb_exCP
             ld a,REPRATE       ; hodnota pro repeat
             ld (hl),a
             ld a,(kb_key)
             jp kb_setcode      ; opakovani vstupu

kb_exCP:     ld a,KB_CP
             ld (kb_count),a    ; pocitadlo shody od zacatku
             ret

kb_scan:     ld bc, 08FEh       ; 8 sloupcu, maska
cols_cyc:    out0 (_PIODB), c   ; buzeni sloupce
             in0 a, (_PIODA)    ; cteni radku
             cpl a            
             or a
             jr nz,kbc_found    ; nalezena klavesa
             rrc c;
             djnz cols_cyc
             xor a,a
             ret
kbc_found:   ld c,0
kbc_code:    inc c
             rrc a
             jr nc,kbc_code    ; cislo radku podle masky
             dec c
             ld a,b
             sla a
             sla a
             sla a
             or a,c             ; a cislo sloupce
             ret
#endasm


main () {
  int i;
  init_keyboard();
  for (;;) {
    printf("\r%x ",kb_key&0xFF);    // indikace stlaceni
    if(kb_code) {
      // ... zpracovani klavesy
        printf("\a");               // pipnuti pri nastaveni kb_code  
      kb_code=0;                    // uvolneni pro dalsi klavesu
    }
  }
}

