
/*****************************************************************

         PLCDCLK2.C

         Promos SBPS 
            with PKDM, character display and real-time clock


    Make a very expensive digital clock.
    Shows how to read the real-time clock and output strings to
    the PKDM

*****************************************************************/
#define LK_LINES 4
#define LK_COLS 20

#use "pmscpu.lib"
#use "pkdm.lib"

#define PKDM 0x8C     // module address


struct tm dt;

char *days[]={"Sunday", "Monday", "Tuesday","Wednesday","Thursday",
               "Friday","Saturday"};

char *months[]={"January", "Febuary", "March", "April", "May", "June", 
                "July", "August","September", "October", "November", 
                "December"};

main ()
{
    int k, ok, start_pos, led_on, last_key;
    char old_day, old_sec, old_hour;
    char strbuf [LK_COLS+1];

    old_day = 0;     /* init to zero to get first display */
    old_hour = 0;
    old_sec = 0;
    led_on = 4;
    k = -2;
    ok=!lk_init_keypad (PKDM);// initialize and blank LCD, returns 0 if OK
    lk_printf ("\x1b0");      // turn off cursor */
    lk_setled(0x88);          // set LED RUN flash
    lk_stdcg();
    while (ok) {
      if ((tm_rd (&dt)) != 0) { // make sure the clock is there 
          lk_pos (0,0);
          lk_printf ("Clock not available");
          lk_setbeep (1000);
          ok = 0;
          lk_setled(0x44);  // set Error LED flash
      }
      else {
          if (old_day != dt.tm_mday) { /* has the date changed? */
             start_pos = (LK_COLS - strlen (days [dt.tm_wday])) / 2;
             lk_pos (0, start_pos);
             lk_printf ("\x1bc%s", days [dt.tm_wday]);
             sprintf (strbuf, "%s %d, %d", months [dt.tm_mon - 1], 
                        dt.tm_mday, dt.tm_year + 1900);
             start_pos = (LK_COLS - strlen (strbuf)) / 2;
             lk_pos (1, start_pos);
             lk_printf ("\x1bc%s", strbuf);  
             old_day = dt.tm_mday;
         }
         if (old_sec != dt.tm_sec) {     /* has the time changed? */
            start_pos = (LK_COLS - 8) / 2;
            lk_pos (2, start_pos);
            lk_printf ("\x1bc"); /* position - line 3 col 6, EOL */
            lk_printf ("%d:%02d:%02d", dt.tm_hour, dt.tm_min, dt.tm_sec);
            old_sec = dt.tm_sec;
            if (old_hour != dt.tm_hour) {
               lk_setbeep (500);
               old_hour = dt.tm_hour;
            }
            else lk_setbeep (105);

            lk_setled(0x88);      // off LED mode
            led_on = (led_on==1) ? 2 : 1;
            lk_setled(0x88+led_on);
            k=lk_kxget(0);
            if(k!=-1) {
              lk_pos(3,1);
              lk_printf ("\x1bcKey code:  %x", k);  
            }
            else { lk_pos(3,1); lk_printf("\x1bc"); }
         }
      }     /* end if clock */
    }       /* end of while loop */
   return -1;
}

/*************************************************End of PLCDCLK2.C */
