/* ======= Promos =====================================================
 Demonstrace pouziti vnitrniho timeru Z180.
   Nastavi timer na 10Hz (100ms) a v prerusovaci procedure inkrementuje
   citac. Dolni byte citace je indikova na LED portu PC.
   Muzete si vyzkouset, jak Dynamic C zachazi s breakpointem, umistenym v
   hlavni smycce a v prerusovaci rutine. DC rozlisuje "obycejny" break (F2)
   (preruseni je nadale povoleno) a "tvrdy" break (ALT+F2) - zastavi 
   preruseni. INT flag je mozne ovladat ALT+I. 
      
 Nastaveni propojek:
   Propojka X7 - spojeny piny 2-3 (port PC vystupni)
*/

#use "pmscpu.lib"

#define TDE1      1           // PRT ch 1 down count enable bit 
#define TIE1      5           // PRT ch 1 interrupt enable bit  

shared int counter;           // sdileny citac 

main() {
   counter = 0x0;                  // pocatecni stav citace
   prt1_init (46080U);             // (9216000/20) / 4608 = 10 Hz
   for( ; counter<512; ) 
   printf("\rCounter:%6d", counter); // pouze vypis
   IRES(TCR,TDE1); IRES(TCR,TIE1); // konec citani
   outport(_PCD,0xFF);
   puts("\rCitani ukonceno.");
}

#INT_VEC PRT1_VEC prt1_isr         // vektor pro PRT1  

interrupt ret int prt1_isr ()  {
   inport(TCR); inport(TMDR1L);    // clear TIF 
   counter++;                      // inkrement citace
   outport(_PCD,~counter);         // sviti bity, ktere jsou 0
}

int prt1_init (int tc) {
   IRES (TCR,TDE1); IRES (TCR,TIE1);           // zastaveni
   outport(TMDR1L,tc); outport(TMDR1H,tc>>8);  // nastaveni konstanty citace
   outport(RLDR1L,tc); outport(RLDR1H,tc>>8);  // nastaveni reload reg.
   ISET (TCR,TDE1); ISET (TCR,TIE1);           // start
}

