/* ====== Promos =========================================================
 Demonstracni program generace casovych intervalu
   Program pouziva vsechny casovace CTC obvodu KIO a dva interni timery
   Z180. Delici pomer kazdeho citace je o drobnou cast rozdilny, takze 
   na LED portu PC je mozne pozorovat zazneje jednotlivych kmitoctu.
   Vsechny casovace jsou obsluhovany pod prerusenim, kazdy casovac ma 
   vlastni vektor a vlastni obluznou prerusovaci rutinu.
   Casovace CTC maji delici pomer 255,254,253 a 252 z taktoveho kmitoctu/256 
   a prerusovaci rutina obsahuje dalsi delic 20. Celkovy pomer je tedy 
   256*255*20 tj. vysledkem je asi 7 Hz. Casovace PRT jsou odvozeny od 
   taktoveho kmitoctu/20, takze jejich obsluzna rutina neobsahuje
   dalsi delici pomer. Nejmensi celkova perioda zazneje je neco pres minutu.
    
   Program dokazuje, ze se preruseni od ruznych zdroju navzajem nijak
   nerusi. Za 1 s je obslouzeno asi 850 preruseni od vsech 6 casovacu,
   coz sice neni tak mnoho, ale vzajemne zazneje zajistuji, ze se jiste 
   najde soucasny vyskyt zadosti o preruseni jak z jednotlivych KIO-CTC 
   vzajemne, tak i s internimi casovaci Z180. 
  
   Za povsimnuti stoji obsluha timeru Z180 - dulezite je shozeni TIF flagu,
   ktery zrusi zadost o preruseni. Obsluha preruseni z CTC nevyzaduje
   zadne usili, zruseni zadosti obstara arbitr KIO ihned po detekci INTA,
   dalsi preruseni z KIO muze prijit po detekci RETI, kterym se prerusovaci 
   procedura ukoncuje (zajistuje klicove slovo reti v deklaraci). Obsluzne
   rutiny PRT nemusi mit uvedeno reti, protoze obsluha internich zdroju
   Z180 se detekuje vnitrnim mechanismem zpracovani preruseni. 
*/

#use "pmscpu.lib"

char x;              // promenna pro ovladani LED
char ctc0_count,     // dodatecne citace pro deleni v prerusovaci rutine
     ctc1_count,     //
     ctc2_count,     //
     ctc3_count;     //

main()
{
   x=0xFF;               // zhasnout LED
   ctc0_count=ctc1_count=ctc2_count=ctc3_count=20;  //pocatecni stavy delicu
   setdaisy ( 3 );       // nastaveni arbitru KIO
   DI();
   setctc (0,1,255,1);   // init CTC0 - sysclock/256, :255, INT
   setctc (1,1,254,1);   // init CTC0 - sysclock/256, :254, INT
   setctc (2,1,253,1);   // init CTC0 - sysclock/256, :253, INT
   setctc (3,1,252,1);   // init CTC0 - sysclock/256, :252, INT
   prt0_init (251*256);  // init PRT0 - sysclock/20,  :251*256, INT
   prt1_init (250*256);  // init PRT1 - sysclock/20,  :250*256, INT
   EI();

   while(1)              
      outport(_PCD,x);    // obnova LED
}

#INT_VEC _CTC0_VEC ctc0_isr    // vektory pro kazdy casovac 
#INT_VEC _CTC1_VEC ctc1_isr
#INT_VEC _CTC2_VEC ctc2_isr
#INT_VEC _CTC3_VEC ctc3_isr
#INT_VEC  PRT0_VEC prt0_isr
#INT_VEC  PRT1_VEC prt1_isr

interrupt reti int ctc0_isr () {
  if(--ctc0_count) return; 
  ctc0_count=20; 
  BIT(&x,0) ? RES(&x,0) : SET(&x,0);
}

interrupt reti int ctc1_isr () {
  if(--ctc1_count) return;
  ctc1_count=20; 
  BIT(&x,1) ? RES(&x,1) : SET(&x,1);
}

interrupt reti int ctc2_isr () {
  if(--ctc2_count) return; 
  ctc2_count=20; 
  BIT(&x,2) ? RES(&x,2) : SET(&x,2);
}

interrupt reti int ctc3_isr () {
  if(--ctc3_count) return; 
  ctc3_count=20; 
  BIT(&x,3) ? RES(&x,3) : SET(&x,3);
}

//------- obsluha internich timeru Z180

interrupt int prt0_isr () {
   inport(TCR); inport(TMDR0L);       // shozeni TIF
   BIT(&x,4) ? RES(&x,4) : SET(&x,4);
}

interrupt int prt1_isr () {
   inport(TCR); inport(TMDR1L);       // shozeni TIF
   BIT(&x,5) ? RES(&x,5) : SET(&x,5);
}






































